==> snapstraction-0.11.0-py2-none-any.whl: tests/test_node_finder.py <==
from collections import OrderedDict
import logging
import tempfile

from tornado.testing import AsyncTestCase, gen_test
from tornado.concurrent import Future

from snapstraction.gus import Gus, NoResponseError

import snap_profile

from tests import helpers
import mock
import unittest

from snapstraction import node_finder


class MockGus(Gus):
    def __new__(cls):
        obj = object.__new__(Gus)
        obj.__init__()
        return obj


def mock_gus():
    gus = mock.MagicMock()
    gus.bridge_address = '000001'
    gus.address = '111110'

    gus.ucast_rpc.return_value = Future()
    gus.ucast_rpc.return_value.set_result(True)

    gus.mcast_rpc.return_value = Future()
    gus.mcast_rpc.return_value.set_result(True)

    gus.dmcast_rpc.return_value = Future()
    gus.dmcast_rpc.return_value.set_result(True)

    gus.callback_ucast_rpc.return_value = Future()
    gus.callback_ucast_rpc.return_value.set_result(True)

    gus.callback_mcast_rpc.return_value = Future()
    gus.callback_mcast_rpc.return_value.set_result(True)

    gus.callback_dmcast_rpc.return_value = Future()
    gus.callback_dmcast_rpc.return_value.set_result(True)

    gus.ucast_data.return_value = Future()
    gus.ucast_data.return_value.set_result(True)

    gus.mcast_data.return_value = Future()
    gus.mcast_data.return_value.set_result(True)

    gus.callback_ucast_data.return_value = Future()
    gus.callback_ucast_data.return_value.set_result(True)

    gus.callback_mcast_data.return_value = Future()
    gus.callback_mcast_data.return_value.set_result(True)

    gus.traceroute.return_value = Future()
    gus.traceroute.return_value.set_result(True)

    gus._base = MockGus()

    gus.on_rpc = gus._base.on_rpc
    gus.on_any_rpc = gus._base.on_any_rpc
    gus.on_data = gus._base.on_data
    gus.on_traceroute = gus._base.on_traceroute

    return gus


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.gus = mock_gus()

        self.target = 'aabbcc'  # The address of the node to interact with
        self.snap_address = '012345'  # The address of the snap belt node
        self.pkt_content = 'foobar'  # The payload of incoming data packets

        self.gus.address = self.snap_address

        super(BaseTestCase, self).setUp()


class NodeFinderTests(AsyncTestCase, BaseTestCase):

    def setUp(self):
        super(NodeFinderTests, self).setUp()

    @gen_test
    def test_ping_target(self):
        yield node_finder.ping_target(self.gus, "012345")
        self.gus.callback_ucast_rpc.assert_called_once_with("012345", 'getNetId')

    @gen_test
    def test_ping_target_does_not_raise_on_failure(self):
        self.gus.callback_ucast_rpc.side_effect = NoResponseError()
        yield node_finder.ping_target(self.gus, 'FFFFFF')
        self.gus.callback_ucast_rpc.assert_any_call('FFFFFF', 'getNetId')

    @gen_test
    def test_ping_on_channel_bridge(self):
        yield node_finder.ping_on_channel(self.gus, self.gus.bridge_address, 0)

        self.gus.callback_ucast_rpc.assert_has_calls([mock.call(self.gus.bridge_address, 'loadNvParam', 4),
                                                      mock.call(self.gus.bridge_address, 'loadNvParam', 3)])

    @gen_test
    def test_ping_on_channel_non_bridge(self):
        yield node_finder.ping_on_channel(self.gus, "012345", 0)

        self.gus.callback_ucast_rpc.assert_has_calls([mock.call(self.gus.bridge_address, 'setChannel', 0),
                                                      mock.call(self.gus.bridge_address, 'setNetId', 0xFFFF),
                                                      mock.call("012345", 'getNetId')])

    def test_move_channel_to_front(self):
        for i in xrange(16):
            self.assertNotEqual(node_finder.CHANNELS[0], i)
            node_finder.move_channel_to_front(i)
            self.assertEqual(node_finder.CHANNELS[0], i)

    @gen_test
    def test_find_one_target(self):
        with mock.patch('snapstraction.node_finder.ping_on_channel') as ping:
            ping.return_value = helpers.make_resolved_future((4, 0x1c2c))
            result = yield node_finder.find_one_target(self.gus, "012345")
            ping.assert_called_once_with(self.gus, "012345", 4)
            self.assertEqual(result, (4, 0x1c2c))

    @gen_test
    def test_find_one_target_fail(self):
        with mock.patch('snapstraction.node_finder.ping_on_channel') as ping:
            ping.return_value = helpers.make_resolved_future((None, None))
            with self.assertRaises(Exception):
                yield node_finder.find_one_target(self.gus, "012345")

    @gen_test
    def test_move_target_to_network(self):

        network = snap_profile.network.Network()

        with mock.patch('snapstraction.node_finder.check_and_set_nvparam') as nvset:
            nvset.side_effect = lambda g, t, k, v: helpers.make_resolved_future((v,))

            self.gus.callback_ucast_rpc.return_value = helpers.make_resolved_future((0x001F,))

            data = yield node_finder.move_target_to_network(self.gus, '012345', network)

        self.assertEqual(data['012345']['to_channel'], network.as_sc_params().channel)
        self.assertEqual(data['012345']['to_net_id'], "0x%04x" % network.as_sc_params().net_id)
        self.assertEqual(data['012345']['to_enc_key'], network.as_sc_params().encryption_key)
        self.assertEqual(data['012345']['to_enc_type'], network.as_sc_params().encryption_type)
        self.assertEqual(data['012345']['to_features'], "0x%04x" % (network.as_sc_params().feature_bits + 0x1f))

    @gen_test
    def test_finder_single_target(self):
        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            data = yield node_finder.finder(self.gus, self.gus.bridge_address)

        self.assertEqual(data[self.gus.bridge_address]['address'], self.gus.bridge_address)
        self.assertEqual(data[self.gus.bridge_address]['on_channel'], 4)
        self.assertEqual(data[self.gus.bridge_address]['on_net_id'], '0x1c2c')

    @gen_test
    def test_finder_fail_to_find_single_target(self):
        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.side_effect = Exception()

            data = yield node_finder.finder(self.gus, self.gus.bridge_address)

        self.assertEqual(data[self.gus.bridge_address], {})

    @gen_test
    def test_get_targets_from_file(self):
        fd, path = tempfile.mkstemp()
        with open(path, 'wt') as f:
            f.write("""
012345
987654
# This is a comment
# 0DECAF
ABCDEF
# This line has spaces
    000001\t\t
""")
        targets = node_finder.get_targets_from_file(path)

        self.assertEqual(targets, ['012345', '987654', 'ABCDEF', '000001'])

    @gen_test
    def test_finder_find_multiple_targets_by_list(self):
        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            data = yield node_finder.finder(self.gus, ['012345', '987654'])

        self.assertEqual(data['012345']['address'], '012345')
        self.assertEqual(data['012345']['on_channel'], 4)
        self.assertEqual(data['012345']['on_net_id'], '0x1c2c')

        self.assertEqual(data['987654']['address'], '987654')
        self.assertEqual(data['987654']['on_channel'], 4)
        self.assertEqual(data['987654']['on_net_id'], '0x1c2c')

    @gen_test
    def test_finder_find_multiple_targets_from_file(self):
        fd, path = tempfile.mkstemp()
        with open(path, 'wt') as f:
            f.write("""
012345
987654
""")

        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            targets = node_finder.get_targets_from_file(path)
            data = yield node_finder.finder(self.gus, targets)

        self.assertEqual(data['012345']['address'], '012345')
        self.assertEqual(data['012345']['on_channel'], 4)
        self.assertEqual(data['012345']['on_net_id'], '0x1c2c')

        self.assertEqual(data['987654']['address'], '987654')
        self.assertEqual(data['987654']['on_channel'], 4)
        self.assertEqual(data['987654']['on_net_id'], '0x1c2c')

    @gen_test
    def test_finder_find_and_move_single_target(self):
        network = snap_profile.network.Network()

        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            with mock.patch('snapstraction.node_finder.check_and_set_nvparam') as nvset:
                nvset.side_effect = lambda g, t, k, v: helpers.make_resolved_future((v,))

                self.gus.callback_ucast_rpc.return_value = helpers.make_resolved_future((0x001F,))

                data = yield node_finder.finder(self.gus, 'bridge', move_to_network=network)

        logging.debug(data)

        self.assertEqual(data[self.gus.bridge_address]['address'], self.gus.bridge_address)
        self.assertEqual(data[self.gus.bridge_address]['on_channel'], 4)
        self.assertEqual(data[self.gus.bridge_address]['on_net_id'], '0x1c2c')
        self.assertEqual(data[self.gus.bridge_address]['to_channel'], network.as_sc_params().channel)
        self.assertEqual(data[self.gus.bridge_address]['to_net_id'], "0x%04x" % network.as_sc_params().net_id)
        self.assertEqual(data[self.gus.bridge_address]['to_enc_key'], network.as_sc_params().encryption_key)
        self.assertEqual(data[self.gus.bridge_address]['to_enc_type'], network.as_sc_params().encryption_type)
        self.assertEqual(data[self.gus.bridge_address]['to_features'], "0x%04x" % (network.as_sc_params().feature_bits + 0x1f))

    @gen_test
    def test_finder_fail_to_find_one_target_in_multiple(self):
        fd, path = tempfile.mkstemp()
        with open(path, 'wt') as f:
            f.write("""
012345
987654
""")

        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.side_effect = [Exception(), helpers.make_resolved_future((4, 0x1c2c))]

            targets = node_finder.get_targets_from_file(path)
            data = yield node_finder.finder(self.gus, targets)

        self.assertEqual(data['012345'], {})

        self.assertEqual(data['987654']['address'], '987654')
        self.assertEqual(data['987654']['on_channel'], 4)
        self.assertEqual(data['987654']['on_net_id'], '0x1c2c')

==> snapstraction-1.1.0-py2-none-any.whl: tests/test_node_finder.py <==
from collections import OrderedDict
import logging
import tempfile

from tornado.testing import AsyncTestCase, gen_test
from tornado.concurrent import Future

from snapstraction.gus import Gus, NoResponseError

import snap_profile

from tests import helpers
import mock
import unittest

from snapstraction import node_finder


class MockGus(Gus):
    def __new__(cls):
        obj = object.__new__(Gus)
        obj.__init__()
        return obj


def mock_gus():
    gus = mock.MagicMock()
    gus.bridge_address = '000001'
    gus.address = '111110'

    gus.ucast_rpc.return_value = Future()
    gus.ucast_rpc.return_value.set_result(True)

    gus.mcast_rpc.return_value = Future()
    gus.mcast_rpc.return_value.set_result(True)

    gus.dmcast_rpc.return_value = Future()
    gus.dmcast_rpc.return_value.set_result(True)

    gus.callback_ucast_rpc.return_value = Future()
    gus.callback_ucast_rpc.return_value.set_result(True)

    gus.callback_mcast_rpc.return_value = Future()
    gus.callback_mcast_rpc.return_value.set_result(True)

    gus.callback_dmcast_rpc.return_value = Future()
    gus.callback_dmcast_rpc.return_value.set_result(True)

    gus.ucast_data.return_value = Future()
    gus.ucast_data.return_value.set_result(True)

    gus.mcast_data.return_value = Future()
    gus.mcast_data.return_value.set_result(True)

    gus.callback_ucast_data.return_value = Future()
    gus.callback_ucast_data.return_value.set_result(True)

    gus.callback_mcast_data.return_value = Future()
    gus.callback_mcast_data.return_value.set_result(True)

    gus.traceroute.return_value = Future()
    gus.traceroute.return_value.set_result(True)

    gus._base = MockGus()

    gus.on_rpc = gus._base.on_rpc
    gus.on_any_rpc = gus._base.on_any_rpc
    gus.on_data = gus._base.on_data
    gus.on_traceroute = gus._base.on_traceroute

    return gus


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.gus = mock_gus()

        self.target = 'aabbcc'  # The address of the node to interact with
        self.snap_address = '012345'  # The address of the snap belt node
        self.pkt_content = 'foobar'  # The payload of incoming data packets

        self.gus.address = self.snap_address

        super(BaseTestCase, self).setUp()


class NodeFinderTests(AsyncTestCase, BaseTestCase):

    def setUp(self):
        super(NodeFinderTests, self).setUp()

    @gen_test
    def test_ping_target(self):
        yield node_finder.ping_target(self.gus, "012345")
        self.gus.callback_ucast_rpc.assert_called_once_with("012345", 'getNetId')

    @gen_test
    def test_ping_target_does_not_raise_on_failure(self):
        self.gus.callback_ucast_rpc.side_effect = NoResponseError()
        yield node_finder.ping_target(self.gus, 'FFFFFF')
        self.gus.callback_ucast_rpc.assert_any_call('FFFFFF', 'getNetId')

    @gen_test
    def test_ping_on_channel_bridge(self):
        yield node_finder.ping_on_channel(self.gus, self.gus.bridge_address, 0)

        self.gus.callback_ucast_rpc.assert_has_calls([mock.call(self.gus.bridge_address, 'loadNvParam', 4),
                                                      mock.call(self.gus.bridge_address, 'loadNvParam', 3)])

    @gen_test
    def test_ping_on_channel_non_bridge(self):
        yield node_finder.ping_on_channel(self.gus, "012345", 0)

        self.gus.callback_ucast_rpc.assert_has_calls([mock.call(self.gus.bridge_address, 'setChannel', 0),
                                                      mock.call(self.gus.bridge_address, 'setNetId', 0xFFFF),
                                                      mock.call("012345", 'getNetId')])

    def test_move_channel_to_front(self):
        for i in xrange(16):
            self.assertNotEqual(node_finder.CHANNELS[0], i)
            node_finder.move_channel_to_front(i)
            self.assertEqual(node_finder.CHANNELS[0], i)

    @gen_test
    def test_find_one_target(self):
        with mock.patch('snapstraction.node_finder.ping_on_channel') as ping:
            ping.return_value = helpers.make_resolved_future((4, 0x1c2c))
            result = yield node_finder.find_one_target(self.gus, "012345")
            ping.assert_called_once_with(self.gus, "012345", 4)
            self.assertEqual(result, (4, 0x1c2c))

    @gen_test
    def test_find_one_target_fail(self):
        with mock.patch('snapstraction.node_finder.ping_on_channel') as ping:
            ping.return_value = helpers.make_resolved_future((None, None))
            with self.assertRaises(Exception):
                yield node_finder.find_one_target(self.gus, "012345")

    @gen_test
    def test_move_target_to_network(self):

        network = snap_profile.network.Network()

        with mock.patch('snapstraction.node_finder.check_and_set_nvparam') as nvset:
            nvset.side_effect = lambda g, t, k, v: helpers.make_resolved_future((v,))

            self.gus.callback_ucast_rpc.return_value = helpers.make_resolved_future((0x001F,))

            data = yield node_finder.move_target_to_network(self.gus, '012345', network)

        self.assertEqual(data['012345']['to_channel'], network.as_sc_params().channel)
        self.assertEqual(data['012345']['to_net_id'], "0x%04x" % network.as_sc_params().net_id)
        self.assertEqual(data['012345']['to_enc_key'], network.as_sc_params().encryption_key)
        self.assertEqual(data['012345']['to_enc_type'], network.as_sc_params().encryption_type)
        self.assertEqual(data['012345']['to_second_crc'], False)
        self.assertEqual(data['012345']['to_packet_crc'], False)

    @gen_test
    def test_finder_single_target(self):
        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            data = yield node_finder.finder(self.gus, self.gus.bridge_address)

        self.assertEqual(data[self.gus.bridge_address]['address'], self.gus.bridge_address)
        self.assertEqual(data[self.gus.bridge_address]['on_channel'], 4)
        self.assertEqual(data[self.gus.bridge_address]['on_net_id'], '0x1c2c')

    @gen_test
    def test_finder_fail_to_find_single_target(self):
        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.side_effect = Exception()

            data = yield node_finder.finder(self.gus, self.gus.bridge_address)

        self.assertEqual(data[self.gus.bridge_address], {})

    @gen_test
    def test_get_targets_from_file(self):
        fd, path = tempfile.mkstemp()
        with open(path, 'wt') as f:
            f.write("""
012345
987654
# This is a comment
# 0DECAF
ABCDEF
# This line has spaces
    000001\t\t
""")
        targets = node_finder.get_targets_from_file(path)

        self.assertEqual(targets, ['012345', '987654', 'ABCDEF', '000001'])

    @gen_test
    def test_finder_find_multiple_targets_by_list(self):
        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            data = yield node_finder.finder(self.gus, ['012345', '987654'])

        self.assertEqual(data['012345']['address'], '012345')
        self.assertEqual(data['012345']['on_channel'], 4)
        self.assertEqual(data['012345']['on_net_id'], '0x1c2c')

        self.assertEqual(data['987654']['address'], '987654')
        self.assertEqual(data['987654']['on_channel'], 4)
        self.assertEqual(data['987654']['on_net_id'], '0x1c2c')

    @gen_test
    def test_finder_find_multiple_targets_from_file(self):
        fd, path = tempfile.mkstemp()
        with open(path, 'wt') as f:
            f.write("""
012345
987654
""")

        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            targets = node_finder.get_targets_from_file(path)
            data = yield node_finder.finder(self.gus, targets)

        self.assertEqual(data['012345']['address'], '012345')
        self.assertEqual(data['012345']['on_channel'], 4)
        self.assertEqual(data['012345']['on_net_id'], '0x1c2c')

        self.assertEqual(data['987654']['address'], '987654')
        self.assertEqual(data['987654']['on_channel'], 4)
        self.assertEqual(data['987654']['on_net_id'], '0x1c2c')

    @gen_test
    def test_finder_find_and_move_single_target(self):
        network = snap_profile.network.Network()

        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.return_value = helpers.make_resolved_future((4, 0x1c2c))

            with mock.patch('snapstraction.node_finder.check_and_set_nvparam') as nvset:
                nvset.side_effect = lambda g, t, k, v: helpers.make_resolved_future((v,))

                self.gus.callback_ucast_rpc.return_value = helpers.make_resolved_future((0x001F,))

                data = yield node_finder.finder(self.gus, 'bridge', move_to_network=network)

        logging.debug(data)

        self.assertEqual(data[self.gus.bridge_address]['address'], self.gus.bridge_address)
        self.assertEqual(data[self.gus.bridge_address]['on_channel'], 4)
        self.assertEqual(data[self.gus.bridge_address]['on_net_id'], '0x1c2c')
        self.assertEqual(data[self.gus.bridge_address]['to_channel'], network.as_sc_params().channel)
        self.assertEqual(data[self.gus.bridge_address]['to_net_id'], "0x%04x" % network.as_sc_params().net_id)
        self.assertEqual(data[self.gus.bridge_address]['to_enc_key'], network.as_sc_params().encryption_key)
        self.assertEqual(data[self.gus.bridge_address]['to_enc_type'], network.as_sc_params().encryption_type)
        self.assertEqual(data[self.gus.bridge_address]['to_second_crc'], False)
        self.assertEqual(data[self.gus.bridge_address]['to_packet_crc'], False)

    @gen_test
    def test_finder_fail_to_find_one_target_in_multiple(self):
        fd, path = tempfile.mkstemp()
        with open(path, 'wt') as f:
            f.write("""
012345
987654
""")

        with mock.patch('snapstraction.node_finder.find_one_target') as find_one:
            find_one.side_effect = [Exception(), helpers.make_resolved_future((4, 0x1c2c))]

            targets = node_finder.get_targets_from_file(path)
            data = yield node_finder.finder(self.gus, targets)

        self.assertEqual(data['012345'], {})

        self.assertEqual(data['987654']['address'], '987654')
        self.assertEqual(data['987654']['on_channel'], 4)
        self.assertEqual(data['987654']['on_net_id'], '0x1c2c')
